@@ -1,11 +1,13 @@
 # encoding: utf-8
-import re
 from collections import OrderedDict
 from importlib import import_module
 
 from django.conf import settings
-from django.core.validators import BaseValidator, ProhibitNullCharactersValidator
+from django.core.validators import (
+    BaseValidator, ProhibitNullCharactersValidator,
+)
 from django.db import models
+from django.db.models import NOT_PROVIDED
 from django.shortcuts import render
 from django.template.loader import render_to_string
 from django.utils.translation import gettext as _
@@ -18,11 +20,32 @@ from cool.views.error_code import ErrorCode
 from cool.views.view import CoolBFFAPIView
 
 
+def parse_validation_error(data):
+    from django.core.exceptions import ValidationError
+    from rest_framework.exceptions import (
+        ValidationError as RestValidationError,
+    )
+    if isinstance(data, ValidationError):
+        if hasattr(data, 'error_dict'):
+            return parse_validation_error(dict(data))
+        return parse_validation_error(list(data))
+    elif isinstance(data, RestValidationError):
+        return parse_validation_error(data.detail)
+    elif isinstance(data, dict):
+        return {key: parse_validation_error(value) for key, value in data.items()}
+    elif isinstance(data, list):
+        return [parse_validation_error(item) for item in data]
+    else:
+        return data
+
+
 def get_rest_field_from_model_field(model, model_field, **kwargs):
     if isinstance(model_field, models.Field):
         model_field = model_field.name
     s = ModelSerializer()
-    field_class, field_kwargs = s.build_field(model_field, model_meta.get_field_info(model), model, 0)
+    info = model_meta.get_field_info(model)
+    field_info = info.fields_and_pk[model_field]
+    field_class, field_kwargs = s.build_field(model_field, info, model, 0)
     field_kwargs.pop('read_only', None)
     gen_validators = field_kwargs.pop('validators', None)
     if gen_validators:
@@ -35,6 +58,15 @@ def get_rest_field_from_model_field(model, model_field, **kwargs):
         if gen_validators:
             field_kwargs['validators'] = gen_validators
     field_kwargs = s.include_extra_kwargs(field_kwargs, kwargs)
+    if not field_kwargs.get('required') and 'default' not in field_kwargs:
+        field_kwargs['default'] = None if field_info.default is NOT_PROVIDED else field_info.default
+
+    if not ('default' in kwargs or 'required' in kwargs or field_kwargs.get('required') or field_kwargs['default']):
+        if (not field_kwargs.get('allow_null', False) and field_kwargs['default'] is None) \
+                or (not field_kwargs.get('allow_blank', False) and not field_kwargs['default']):
+            field_kwargs['required'] = True
+            field_kwargs.pop('default')
+
     return field_class(**field_kwargs)
 
 
@@ -55,13 +87,12 @@ def get_field_info(field):
     field_validators = [field]
     field_validators.extend(getattr(field, 'validators', list()))
     validator_keys = ['max_value', 'min_value', 'max_length', 'min_length', 'max_digits', 'max_decimal_places',
-                      'sep', 'child', 'choices', 'regex', 'allowed_extensions']
+                      'choices', 'regex', 'allowed_extensions', 'sep', 'child', 'is_list', 'children']
     for validator in field_validators:
         for k in validator_keys:
             v = getattr(validator, k, None)
             if v is not None:
-                if isinstance(v, re.Pattern):
-                    v = v.pattern
+                v = getattr(v, 'pattern', v)
                 info['extend_info'].setdefault(k, list()).append(v)
         if isinstance(validator, BaseValidator):
             info['extend_info'].setdefault(validator.code, list()).append(validator.limit_value)
@@ -84,7 +115,16 @@ def get_field_info(field):
         if isinstance(_value, list):
             return ",".join(map(lambda x: _format_value(x) if isinstance(x, dict) else x, _value))
         elif isinstance(_value, dict):
-            return ",".join(["%s:%s" % (_k, _v) for _k, _v in _value.items()])
+            return ",".join(["%s:%s" % (_k, _format_value(_v)) for _k, _v in _value.items()])
+        elif isinstance(_value, fields.Field):
+            _info = get_field_info(_value)
+            return "(%s %s 默认值:%s,是否必填:%s,%s)" % (
+                _info['label'],
+                _info['type'],
+                _info['default_format'],
+                _info['required_format'],
+                _info['extend_info_format']
+            )
         else:
             return _value
 
@@ -216,6 +256,7 @@ def get_api_doc_html(request, *args, **kwargs):
     exclude_base_view_params = kwargs.get('exclude_base_view_params', True)
     exclude_views = kwargs.get('exclude_views', ())
     title = kwargs.get('title', _('Api Document'))
+    toc_left = kwargs.get('toc_left', True)
     md = get_api_doc(
         request=request,
         template_name=md_template_name,
@@ -231,4 +272,4 @@ def get_api_doc_html(request, *args, **kwargs):
         'markdown.extensions.tables'
     ])
     md_style_template_name = kwargs.get('md_style_template_name', 'cool/views/markdown.html')
-    return render(request, md_style_template_name, context={'html': html, 'title': title})
+    return render(request, md_style_template_name, context={'html': html, 'title': title, 'toc_left': toc_left})
